function game_load()
	love.graphics.setBackgroundColor(0, 125, 255)
	gamestate = "game"
	blocksize = 16*scale
	gravityswitchtimer = 0
	resetobjects()
	loadmap(currentlevel)
	screenflashalpha = 0

	inputs = {"switch"}
	outputs = {"switchdoor"}

end

function game_update(dt)
	if ending then
		return
	end
	if layertransition then
		oldlayeralpha = oldlayeralpha-400*dt
		newlayeralpha = newlayeralpha+400*dt
		if oldlayeralpha <= 0 or newlayeralpha >= 255 then
			layertransition = false
			oldlayeralpha = 255
			newlayeralpha = 0

			if movelayer == "forward" then
				currentlayer = currentlayer + 1
				for i, v in pairs(objects["player"]) do
					v.layer = currentlayer
				end
				vvvvvvgravity = false
				if layersettings[currentlayer] == "normal" or not layersettings then
					gravity = 20
				elseif layersettings[currentlayer] == "low" then
					gravity = 12
				elseif layersettings[currentlayer] == "high" then
					gravity = 27
				elseif layersettings[currentlayer] == "vvvvvv" then
					vvvvvvgravity = true
					gravity = 20
				end
			elseif movelayer == "backward" then
				currentlayer = currentlayer - 1
				for i, v in pairs(objects["player"]) do
					v.layer = currentlayer
				end
				vvvvvvgravity = false
				if layersettings[currentlayer] == "normal" or not layersettings then
					gravity = 20
				elseif layersettings[currentlayer] == "low" then
					gravity = 12
				elseif layersettings[currentlayer] == "high" then
					gravity = 27
				elseif layersettings[currentlayer] == "vvvvvv" then
					vvvvvvgravity = true
					gravity = 20
				end
			end
		end
		return
	end

	if screenflash then
		screenflashalpha = screenflashalpha + 300*dt
		if screenflashalpha >= 255 then
			screenflashalpha = 0
			screenflash = false
			if refreshlevel then
				refreshlevel = false
				loadmap(currentlevel)
			else
				gotonextlevel = true
			end

		end
		return
	end
	if gotonextlevel then
		gotonextlevel = false
		if love.filesystem.exists("Maps/" .. (currentlevel + 1)) then
			currentlevel = currentlevel + 1
			loadmap(currentlevel)
			return
		else
			ending = true
		end
		
	end
	removeobjects()
	fullentitymap = {}
	for layer = 1, maxlayer do
		fullentitymap[layer] = {}
		for x = 0, mapwidth do
			fullentitymap[layer][x] = {}
		end
	end
	for j, k in pairs(objects) do
		for i, v in pairs(k) do
			fullentitymap[v.layer][math.floor(v.x)][math.floor(v.y)] = {j, i}
		end
	end
	physicsupdate(dt)
	for j, k in pairs(objects) do
		for i, v in pairs(k) do
			if v.update then
				v:update(dt)
			end
		end
	end

	if vvvvvvgravity then
		gravityswitchtimer = gravityswitchtimer + dt
		if gravityswitchtimer > 2 then
			gravityswitchtimer = 0
			gravity = -gravity
		end
	end
end

function game_draw()
	if ending then
		love.graphics.setColor(255, 255, 255)
		love.graphics.draw(backgrounds[4], 0, 0, 0, scale)
		love.graphics.setColor(255, 255, 255)
		love.graphics.print("We hope you enjoyed playing! Remember to give us a rating!", (screenwidth/2)-(font:getWidth("We hope you enjoyed playing! Remember to give us a rating!")/4), screenheight/2-10*scale)
		return
	end

	if not layertransition then
		love.graphics.setBlendMode("alpha")
		scenedraw()
	else
		love.graphics.setBlendMode("alpha")
		love.graphics.setColor(255, 255, 255)
		scenedraw()
		love.graphics.setColor(255, 255, 255, oldlayeralpha)
		love.graphics.rectangle("fill", 0, 0, screenwidth, screenheight)
		love.graphics.setColor(255, 255, 255)
		scenedraw()
	
		love.graphics.setColor(255, 255, 255, newlayeralpha)
		love.graphics.rectangle("fill", 0, 0, screenwidth, screenheight)

		for i, v in pairs(objects["player"]) do
			love.graphics.push()
			love.graphics.translate(-xscroll*blocksize, 0)
			v:draw()
			love.graphics.pop()
		end


	end
	if screenflash then
		love.graphics.setColor(255, 255, 255, screenflashalpha)
		love.graphics.rectangle("fill", 0, 0, screenwidth, screenheight)
		love.graphics.setColor(255, 255, 255, 255)

		for i, v in pairs(objects["player"]) do
			love.graphics.push()
			love.graphics.translate(-xscroll*blocksize, 0)
			v:draw()
			if v.standingonexit then
				v.standingonexit:draw()
			end
			love.graphics.pop()
		end

	end
end

function scenedraw(ispreview)
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(backgrounds[currenttileset], 0, 0, 0, scale)
	love.graphics.push()
	love.graphics.translate(-xscroll*blocksize, 0)

	for i, v in pairs(objects["keydoor"]) do
		if v.layer == currentlayer then
			v:draw()
		end
	end
	for i, v in pairs(objects["switchdoor"]) do
		if v.layer == currentlayer then
			v:draw()
		end
	end
	for i, v in pairs(objects["block"]) do
		if v.layer == currentlayer then
			love.graphics.drawq(tilesets[currenttileset], tilesetquads[v.quad], v.x*blocksize, v.y*blocksize, 0, scale)
		end
	end


	for i, v in pairs(objects["exit"]) do
		if v.layer == currentlayer then
			v:draw()
		end
	end
	for i, v in pairs(objects["key"]) do
		if v.layer == currentlayer then
			v:draw()
		end
	end

	for i, v in pairs(objects["switch"]) do
		if v.layer == currentlayer then
			v:draw()
		end
	end

	for i, v in pairs(objects["player"]) do
		v:draw()
	end
	love.graphics.pop()

	if layersettings[currentlayer] == "high" then
		love.graphics.setColor(255, 0, 0, 50)
		love.graphics.rectangle("fill", 0, 0, screenwidth, screenheight)
	elseif layersettings[currentlayer] == "low" then
		love.graphics.setColor(0, 0, 255, 50)
		love.graphics.rectangle("fill", 0, 0, screenwidth, screenheight)
	end
	if not ispreview then
		inpreview = true 
		love.graphics.setColor(255, 255, 255)
		love.graphics.setLineWidth(2*scale)



		local v1 = objects["player"][1]
		local x, y, width, height = v1.x*blocksize, v1.y*blocksize, v1.width*blocksize, v1.height*blocksize
		if not aabb(x, y, width, height, 25*scale, 25*scale, 80*scale, 50*scale) then
			love.graphics.rectangle("line", 25*scale, 25*scale, 80*scale, 50*scale)
			love.graphics.setColor(love.graphics.getBackgroundColor())
			love.graphics.rectangle("fill", 26*scale, 26*scale, 79*scale, 49*scale)
			love.graphics.setScissor(25*scale, 25*scale, 80*scale, 50*scale)
					love.graphics.setColor(255, 255, 255)
			if currentlayer > 1 then
				currentlayer = currentlayer-1
				love.graphics.push()
				love.graphics.translate(25.5*scale, 26.5*scale)
				love.graphics.scale(.2, .2)
				scenedraw(true)
				love.graphics.pop()
				currentlayer = currentlayer+1
			else

				love.graphics.draw(noiseimage, 25*scale, 25*scale, 0, scale)
				for x = 1, 5 do
					love.graphics.draw(noiseimage, math.random(0*scale, 105*scale), math.random(10*scale, 40*scale), 0, scale)
				end
			end
			love.graphics.setColor(255, 255, 255)

			love.graphics.setScissor()
		end
		if not aabb(x, y, width, height, 295*scale, 25*scale, 80*scale, 50*scale) then
			love.graphics.rectangle("line", 295*scale, 25*scale, 80*scale, 50*scale)
			love.graphics.setColor(love.graphics.getBackgroundColor())
			love.graphics.rectangle("fill", 296*scale, 26*scale, 79*scale, 49*scale)
			love.graphics.setScissor(295*scale, 25*scale, 80*scale, 50*scale)


			love.graphics.setColor(255, 255, 255)
			if currentlayer < maxlayer then
				currentlayer = currentlayer+1
				love.graphics.push()
				love.graphics.translate(295.5*scale, 26.5*scale)
				love.graphics.scale(.2, .2)
				scenedraw(true)
				love.graphics.pop()
				currentlayer = currentlayer-1
			else
				love.graphics.draw(noiseimage, 295*scale, 25*scale, 0, scale)
				for x = 1, 5 do
					love.graphics.draw(noiseimage, math.random(270*scale, 375*scale), math.random(10*scale, 40*scale), 0, scale)
				end
			end
			love.graphics.setScissor()
		end
		love.graphics.setLineWidth(1)
		inpreview = false

	end
end

function game_keypressed(key)
	if ending then
		return
	end
	for i, v in pairs(objects["player"]) do
		v:keypressed(key)
	end
end

function game_mousepressed(x, y, button)
end

function removeobjects()
	for x = #objects["key"], 1, -1 do
		if objects["key"][x].kill then
			table.remove(objects["key"], x)
			playsound(getkeysound)
		end
	end
	for x = #objects["keydoor"], 1, -1 do
		if objects["keydoor"][x].kill then
			table.remove(objects["keydoor"], x)
		end
	end
end

function loadmap(mapnumber)
	oldlayeralpha = 255
	newlayeralpha = 0
	xscroll = 0
	resetobjects()
	fullentitymap = {}
	local numberoflayers = love.filesystem.enumerate("Maps/" .. mapnumber)
	for x = #numberoflayers, 1, -1 do
		if numberoflayers[x] == ".DS_Store" or numberoflayers[x] == "Thumbs.db" or numberoflayers[x] == "info.lua" then
			table.remove(numberoflayers, x)
		end
	end
	for layer = 1, #numberoflayers do
		fullentitymap[layer] = {}
		local mapimage = love.image.newImageData("Maps/" .. mapnumber .. "/" .. layer .. ".png")
		local imagemapwidth, imagemapheight = mapimage:getWidth(), mapimage:getHeight()
		for x = 0, imagemapwidth-1 do
			fullentitymap[layer][x] = {}
			for y = 0, imagemapheight-1 do
				local r, g, b, a = mapimage:getPixel(x, y)
				getmapentity(x, y, r, g, b, a, layer, mapimage)
			end
		end
		mapwidth = imagemapwidth+1
	end

	maxlayer = #numberoflayers
	if love.filesystem.exists("Maps/" .. mapnumber .. "/info.lua") then
		local info = love.filesystem.load("Maps/" .. mapnumber .. "/info.lua")()
		if info.tileset then
			currenttileset = info.tileset
		end
		layersettings = {}
		if info.layersettings then
			for x = 1, maxlayer do 
				layersettings[x] = info.layersettings[x]
			end
		end
		vvvvvvgravity = false
		if layersettings[currentlayer] == "normal" or not layersettings then
			gravity = 20
		elseif layersettings[currentlayer] == "low" then
			gravity = 12
		elseif layersettings[currentlayer] == "high" then
			gravity = 27
		elseif layersettings[currentlayer] == "vvvvvv" then
			gravity = 20
			vvvvvvgravity = true
		end


		if info.links then
			for x = 1, #info.links do

				local data = info.links[x]
				local input
				for j, k in pairs(inputs) do
					for i, v in pairs(objects[k]) do
						if v.x == data[1].x and v.y == data[1].y and v.layer == data[1].layer then
							input = v
						end
					end
				end

				for y = 2, #data do
					for j, k in pairs(outputs) do
						for i, v in pairs(objects[k]) do
							if v.x == data[y].x and v.y == data[y].y and v.layer == data[y].layer then
								v.linkedinput = input
							end
						end
					end
				end

			end
		end
	end
end

function resetobjects()
	objects = {}
	objects["block"] = {}
	objects["player"] = {}
	objects["exit"] = {}
	objects["key"] = {}
	objects["keydoor"] = {}
	objects["switch"] = {}
	objects["switchdoor"] = {}
end

function getmapentity(x, y, r, g, b, a, layer, mapimage)
	if (r == 0 and g == 0 and b == 0 and a > 127) then 
		determineblock(x, y, r, g, b, a, layer, mapimage)

	elseif (r == 0 and g == 255 and b == 0) then
		currentlayer = layer
		table.insert(objects["player"], player:new(x+1-8/16, y-1, layer))

		table.insert(objects["exit"], exit:new(x, y+1, layer, true))
	elseif (r == 0 and g == 0 and b == 255) then
		table.insert(objects["exit"], exit:new(x, y, layer))
	elseif (r == 255 and g == 255 and b == 0) then
		table.insert(objects["key"], key:new(x, y, layer))
	elseif (r==127 and g == 127 and b == 127) then
		table.insert(objects["keydoor"], keydoor:new(x, y, layer))
	elseif (r==50 and g == 50 and b == 50) then
		table.insert(objects["switch"], switch:new(x, y, layer))
	elseif (r==100 and g==100 and b==100) then
		table.insert(objects["switchdoor"], switchdoor:new(x, y, layer))
	end
end

function determineblock(x, y, r, g, b, a, layer, mapimage)
	--LEVEL GENERATION
--COVER YOUR EYES CHILDREN
	local quad = 16
	local topleft, topcenter, topright, middleleft, middleright, bottomleft, bottomcenter, bottomright
	local width, height = mapimage:getWidth(), mapimage:getHeight()
	if x-1 >= 0 and y-1 >= 0 then
		local newr, newg, newb, newa = mapimage:getPixel(x-1, y-1)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			topleft = true
		end
	else
		topleft = true
	end
	if y-1 >= 0 then
		local newr, newg, newb, newa = mapimage:getPixel(x, y-1)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			topcenter = true
		end
	else
		topcenter = true
	end
	if x+1 < width and y-1 >= 0 then
		local newr, newg, newb, newa = mapimage:getPixel(x+1, y-1)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			topright = true
		end
	else
		topright = true
	end
	if x-1 >= 0 then
		local newr, newg, newb, newa = mapimage:getPixel(x-1, y)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			middleleft = true
		end
	else
		middleleft = true
	end
	if x+1 < width then
		local newr, newg, newb, newa = mapimage:getPixel(x+1, y)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			middleright = true
		end
	else
		middleright = true
	end
	if x-1 >= 0 and y+1 < height then
		local newr, newg, newb, newa = mapimage:getPixel(x-1, y+1)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			bottomleft = true
		end
	else
		bottomleft = true
	end
	if y+1 < height then
		local newr, newg, newb, newa = mapimage:getPixel(x, y+1)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			bottomcenter = true
		end
	else
		bottomcenter = true
	end
	if x+1 < width and y+1 < height then
		local newr, newg, newb, newa = mapimage:getPixel(x+1, y+1)
		if (newr == 0 and newg == 0 and newb == 0 and newa > 127) then
			bottomright = true
		end
	else
		bottomright = true
	end

	if (not middleleft and middleright and not topcenter --[[and not topleft and not topright--]] and bottomcenter --[[and not bottomleft and bottomright--]]) then
		quad = 1
	elseif (middleleft and middleright and not topcenter --[[and not topleft and not topright--]] and bottomcenter) then
		quad = 2
	elseif (middleleft and not middleright and not topcenter--[[ and not topleft and not topright--]] and bottomcenter --[[and bottomleft and not bottomright--]]) then
		quad = 3
	elseif (not middleleft and middleright and topcenter --[[and not topleft and topright--]] and bottomcenter --[[and not bottomleft and bottomright--]]) then
		quad = 4
	elseif (middleleft and middleright and topcenter and topleft and topright and bottomcenter and bottomleft and bottomright) then
		quad = 5
	elseif (middleleft and not middleright and topcenter--[[ and topleft and not topright--]] and bottomcenter --[[and bottomleft and not bottomright--]]) then
		quad = 6
	elseif (not middleleft and middleright and topcenter--[[ and not topleft and topright--]] and not bottomcenter --[[and not bottomleft and not bottomright--]]) then
		quad = 7
	elseif (middleleft and middleright and topcenter and topleft and topright and not bottomcenter --[[and not bottomright and not bottomleft--]]) then
		quad = 8
	elseif (middleleft and not middleright and topcenter--[[ and topleft and not topright--]] and not bottomcenter --[[and not bottomleft and not bottomright--]]) then
		quad = 9
	elseif (not middleleft and not middleright and not topcenter and bottomcenter) then
		quad = 10
	elseif (not middleleft and not middleright and topcenter and bottomcenter) then
		quad = 11
	elseif (not middleleft and not middleright and topcenter and not bottomcenter) then
		quad = 12
	elseif (not middleleft and middleright and not topcente) then
		quad = 13
	elseif (middleleft and middleright and not topcenter ) then
		quad = 14
	elseif (middleleft and not middleright and not topcenter ) then
		quad = 15
	elseif (middleright and bottomcenter and not bottomright ) then
		quad = 17
	elseif (middleleft and bottomcenter and not bottomleft) then
		quad = 18
	elseif (middleright and topcenter and not topright) then
		quad = 19
	elseif (middleleft and topcenter and not topleft) then
		quad = 20
	end
	table.insert(objects["block"], block:new(x, y, layer, quad))
end