love.graphics.setDefaultImageFilter("nearest", "nearest")
require "loverun"

require "Classes/class"
require "Classes/block"
require "Classes/player"
require "Classes/exit"
require "Classes/key"
require "Classes/keydoor"
require "Classes/switch"
require "Classes/switchdoor"

require "States/game"
require "States/credits"
require "States/menu"

require "physics"

function love.load()


	gravity = 20
	noiseimage = love.graphics.newImage("Graphics/noise.png")
	scale = 2
	love.graphics.setMode(400*scale, 240*scale)
	currentlevel = 1
	--menu_load()
	game_load()
	screenwidth, screenheight = love.graphics.getWidth(), love.graphics.getHeight()
	screenblockwidth, screenblockwidth = 25, 15
	tilesets = {}
	for x = 1, 5 do 
		tilesets[x] = love.graphics.newImage("Graphics/tiles " .. x .. ".png")
	end
	backgrounds = {}
	for x = 1, 5 do
		backgrounds[x] = love.graphics.newImage("Graphics/background" .. x .. ".png")
	end
	tilesetquads = {}
	local width, height = tilesets[1]:getWidth(), tilesets[1]:getHeight()
	for y = 1, 3 do
		for x = 1, 3 do
			tilesetquads[x+(y-1)*3] = love.graphics.newQuad((x-1)*17, (y-1)*17, 16, 16, width, height)
		end
	end
	for y = 1, 3 do
		tilesetquads[9+y] = love.graphics.newQuad(51, (y-1)*17, 16, 16, width, height)
	end
	for x = 1, 3 do
		tilesetquads[12+x] = love.graphics.newQuad((x-1)*17, 51, 16, 16, width, height)
	end
	tilesetquads[16] = love.graphics.newQuad(51, 51, 16, 16, width, height)

	for y =1, 2 do
		for x = 1, 2 do
			tilesetquads[16+(x+(y-1)*2)] = love.graphics.newQuad(68 + (x-1)*17, (y-1)*17, 16, 16, width, height)
		end
	end

	endlevelsound = love.audio.newSource("Sounds/endlevel.ogg")
	errorsound = love.audio.newSource("Sounds/error.ogg")

	getkeysound = love.audio.newSource("Sounds/get-key.ogg")
	switchsound = love.audio.newSource("Sounds/switch.ogg")

	music = love.audio.newSource("Sounds/heartofmithril.wav")
	music:setLooping(true)
	music:play()

	transitionsound = love.audio.newSource("Sounds/transition.ogg")

	dooropensound = love.audio.newSource("Sounds/door-open.ogg")

	font = love.graphics.newFont(15*scale)
end

function love.update(dt)

	dt = math.min(.2, dt)
	if gamestate == "game" then
		game_update(dt)
	end
	if gamestate == "menu" then
		menu_update(dt)
	end
	if gamestate == "credits" then
		credits_update(dt)
	end
end

function love.draw()
	if gamestate == "game" then
		game_draw()
	end
	if gamestate == "menu" then
		menu_draw()
	end
	if gamestate == "credits" then
		credits_draw()
	end

end

function love.mousepressed(x, y, button)
	if gamestate == "game" then
		game_mousepressed(x, y, button)
	end
end

function love.keypressed(key)
	if gamestate == "game" then
		game_keypressed(key)
	end
end

function playsound(sound)
	love.audio.stop(sound)
	love.audio.play(sound)
end